import sys

from platform import python_version_tuple

from datadog_lambda import __version__
from datadog_lambda.cold_start import get_cold_start_tag


def _format_dd_lambda_layer_tag():
    """
    Formats the dd_lambda_layer tag, e.g., 'dd_lambda_layer:datadog-python39_1'
    """
    runtime = "python{}{}".format(sys.version_info[0], sys.version_info[1])
    return "dd_lambda_layer:datadog-{}_{}".format(runtime, __version__)


def tag_dd_lambda_layer(tags):
    """
    Used by lambda_metric to insert the dd_lambda_layer tag
    """
    dd_lambda_layer_tag = _format_dd_lambda_layer_tag()
    if tags:
        return tags + [dd_lambda_layer_tag]
    else:
        return [dd_lambda_layer_tag]


def parse_lambda_tags_from_arn(lambda_context):
    """Generate the list of lambda tags based on the data in the arn
    Args:
        lambda_context: Aws lambda context object
            ex: lambda_context.arn = arn:aws:lambda:us-east-1:123597598159:function:my-lambda:1
    """
    # Set up flag for extra testing to distinguish between a version or alias
    hasAlias = False
    # Cap the number of times to spli
    split_arn = lambda_context.invoked_function_arn.split(":")

    if len(split_arn) > 7:
        hasAlias = True
        _, _, _, region, account_id, _, function_name, alias = split_arn
    else:
        _, _, _, region, account_id, _, function_name = split_arn

    # Add the standard tags to a list
    tags = [
        "region:{}".format(region),
        "account_id:{}".format(account_id),
        "functionname:{}".format(function_name),
    ]

    # Check if we have a version or alias
    if hasAlias:
        # If $Latest, drop the $ for datadog tag convention. A lambda alias can't start with $
        if alias.startswith("$"):
            alias = alias[1:]
        # Versions are numeric. Aliases need the executed version tag
        elif not check_if_number(alias):
            tags.append("executedversion:{}".format(lambda_context.function_version))
        # create resource tag with function name and alias/version
        resource = "resource:{}:{}".format(function_name, alias)
    else:
        # Resource is only the function name otherwise
        resource = "resource:{}".format(function_name)

    tags.append(resource)

    return tags


def get_runtime_tag():
    """Get the runtime tag from the current Python version"""
    major_version, minor_version, _ = python_version_tuple()

    return "runtime:python{major}.{minor}".format(
        major=major_version, minor=minor_version
    )


def get_library_version_tag():
    """Get datadog lambda library tag"""
    return "datadog_lambda:v{}".format(__version__)


def get_enhanced_metrics_tags(lambda_context):
    """Get the list of tags to apply to enhanced metrics"""
    return parse_lambda_tags_from_arn(lambda_context) + [
        get_cold_start_tag(),
        "memorysize:{}".format(lambda_context.memory_limit_in_mb),
        get_runtime_tag(),
        get_library_version_tag(),
    ]


def check_if_number(alias):
    """
    Check if the alias is a version or number.
    Python 2 has no easy way to test this like Python 3
    """
    try:
        float(alias)
        return True
    except ValueError:
        return False



# The minor version corresponds to the Lambda layer version.
# E.g.,, version 0.5.0 gets packaged into layer version 5.
try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__)

import os
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.getLevelName(os.environ.get("DD_LOG_LEVEL", "INFO").upper()))



from datadog_lambda.stats_writer import StatsWriter
from datadog_lambda.dogstatsd import statsd


class StatsDWriter(StatsWriter):
    """
    Writes distribution metrics using StatsD protocol
    """

    def distribution(self, metric_name, value, tags=[], timestamp=None):
        statsd.distribution(metric_name, value, tags=tags)

    def flush(self):
        pass

    def stop(self):
        pass



import logging
import os
import socket
import errno
import re
from threading import Lock


MIN_SEND_BUFFER_SIZE = 32 * 1024
log = logging.getLogger("datadog_lambda.dogstatsd")


class DogStatsd(object):
    def __init__(self):
        self._socket_lock = Lock()
        self.socket_path = None
        self.host = "localhost"
        self.port = 8125
        self.socket = None
        self.encoding = "utf-8"

    def get_socket(self, telemetry=False):
        """
        Return a connected socket.

        Note: connect the socket before assigning it to the class instance to
        avoid bad thread race conditions.
        """
        with self._socket_lock:
            self.socket = self._get_udp_socket(
                self.host,
                self.port,
            )
            return self.socket

    @classmethod
    def _ensure_min_send_buffer_size(cls, sock, min_size=MIN_SEND_BUFFER_SIZE):
        # Increase the receiving buffer size where needed (e.g. MacOS has 4k RX
        # buffers which is half of the max packet size that the client will send.
        if os.name == "posix":
            try:
                recv_buff_size = sock.getsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF)
                if recv_buff_size <= min_size:
                    sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, min_size)
                    log.debug("Socket send buffer increased to %dkb", min_size / 1024)
            finally:
                pass

    @classmethod
    def _get_udp_socket(cls, host, port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setblocking(0)
        cls._ensure_min_send_buffer_size(sock)
        sock.connect((host, port))

        return sock

    def distribution(self, metric, value, tags=None):
        """
        Send a global distribution value, optionally setting tags.

        >>> statsd.distribution("uploaded.file.size", 1445)
        >>> statsd.distribution("album.photo.count", 26, tags=["gender:female"])
        """
        self._report(metric, "d", value, tags)

    def close_socket(self):
        """
        Closes connected socket if connected.
        """
        with self._socket_lock:
            if self.socket:
                try:
                    self.socket.close()
                except OSError as e:
                    log.error("Unexpected error: %s", str(e))
                self.socket = None

    def normalize_tags(self, tag_list):
        TAG_INVALID_CHARS_RE = re.compile(r"[^\w\d_\-:/\.]", re.UNICODE)
        TAG_INVALID_CHARS_SUBS = "_"
        return [
            re.sub(TAG_INVALID_CHARS_RE, TAG_INVALID_CHARS_SUBS, tag)
            for tag in tag_list
        ]

    def _serialize_metric(self, metric, metric_type, value, tags):
        # Create/format the metric packet
        return "%s:%s|%s%s" % (
            metric,
            value,
            metric_type,
            ("|#" + ",".join(self.normalize_tags(tags))) if tags else "",
        )

    def _report(self, metric, metric_type, value, tags):
        if value is None:
            return

        payload = self._serialize_metric(metric, metric_type, value, tags)

        # Send it
        self._send_to_server(payload)

    def _send_to_server(self, packet):
        try:
            mysocket = self.socket or self.get_socket()
            mysocket.send(packet.encode(self.encoding))
            return True
        except socket.timeout:
            # dogstatsd is overflowing, drop the packets (mimicks the UDP behaviour)
            pass
        except (socket.herror, socket.gaierror) as socket_err:
            log.warning(
                "Error submitting packet: %s, dropping the packet and closing the socket",
                socket_err,
            )
            self.close_socket()
        except socket.error as socket_err:
            if socket_err.errno == errno.EAGAIN:
                log.debug(
                    "Socket send would block: %s, dropping the packet", socket_err
                )
            elif socket_err.errno == errno.ENOBUFS:
                log.debug("Socket buffer full: %s, dropping the packet", socket_err)
            elif socket_err.errno == errno.EMSGSIZE:
                log.debug(
                    "Packet size too big (size: %d): %s, dropping the packet",
                    len(packet.encode(self.encoding)),
                    socket_err,
                )
            else:
                log.warning(
                    "Error submitting packet: %s, dropping the packet and closing the socket",
                    socket_err,
                )
                self.close_socket()
        except Exception as e:
            log.error("Unexpected error: %s", str(e))
        return False


statsd = DogStatsd()



# Unless explicitly stated otherwise all files in this repository are licensed
# under the Apache License Version 2.0.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019 Datadog, Inc.

# Datadog trace sampling priority


class SamplingPriority(object):
    USER_REJECT = -1
    AUTO_REJECT = 0
    AUTO_KEEP = 1
    USER_KEEP = 2


# Datadog trace headers
class TraceHeader(object):
    TRACE_ID = "x-datadog-trace-id"
    PARENT_ID = "x-datadog-parent-id"
    SAMPLING_PRIORITY = "x-datadog-sampling-priority"


# X-Ray subsegment to save Datadog trace metadata
class XraySubsegment(object):
    NAME = "datadog-metadata"
    TRACE_KEY = "trace"
    LAMBDA_FUNCTION_TAGS_KEY = "lambda_function_tags"
    NAMESPACE = "datadog"


# TraceContextSource of datadog context. The DD_MERGE_XRAY_TRACES
# feature uses this to determine when to use X-Ray as the parent
# trace.
class TraceContextSource(object):
    XRAY = "xray"
    EVENT = "event"
    DDTRACE = "ddtrace"


# X-Ray deamon
class XrayDaemon(object):
    XRAY_TRACE_ID_HEADER_NAME = "_X_AMZN_TRACE_ID"
    XRAY_DAEMON_ADDRESS = "AWS_XRAY_DAEMON_ADDRESS"
    FUNCTION_NAME_HEADER_NAME = "AWS_LAMBDA_FUNCTION_NAME"



import logging
from os import path

try:
    # only available in python 3
    # not an issue since the extension is not compatible with python 2.x runtime
    # https://docs.aws.amazon.com/lambda/latest/dg/using-extensions.html
    import urllib.request
except ImportError:
    # safe since both calls to urllib are protected with try/expect and will return false
    urllib = None

AGENT_URL = "http://127.0.0.1:8124"
HELLO_PATH = "/lambda/hello"
FLUSH_PATH = "/lambda/flush"
EXTENSION_PATH = "/opt/extensions/datadog-agent"

logger = logging.getLogger(__name__)


def is_extension_running():
    if not path.exists(EXTENSION_PATH):
        return False
    try:
        urllib.request.urlopen(AGENT_URL + HELLO_PATH)
    except Exception as e:
        logger.debug("Extension is not running, returned with error %s", e)
        return False
    return True


def flush_extension():
    try:
        req = urllib.request.Request(AGENT_URL + FLUSH_PATH, "".encode("ascii"))
        urllib.request.urlopen(req)
    except Exception as e:
        logger.debug("Failed to flush extension, returned with error %s", e)
        return False
    return True


should_use_extension = is_extension_running()



import os
import logging
import json
import binascii
import time
import socket

from datadog_lambda.constants import XrayDaemon, XraySubsegment, TraceContextSource

logger = logging.getLogger(__name__)


def get_xray_host_port(address):
    if address == "":
        logger.debug("X-Ray daemon env var not set, not sending sub-segment")
        return None
    parts = address.split(":")
    if len(parts) <= 1:
        logger.debug("X-Ray daemon env var not set, not sending sub-segment")
        return None
    port = int(parts[1])
    host = parts[0]
    return (host, port)


def send(host_port_tuple, payload):
    sock = None
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setblocking(0)
        sock.connect(host_port_tuple)
        sock.send(payload.encode("utf-8"))
    except Exception as e_send:
        logger.error("Error occurred submitting to xray daemon: %s", str(e_send))
    try:
        sock.close()
    except Exception as e_close:
        logger.error("Error while closing the socket: %s", str(e_close))


def build_segment_payload(payload):
    if payload is None:
        return None
    return '{"format": "json", "version": 1}' + "\n" + payload


def parse_xray_header(raw_trace_id):
    # Example: Root=1-5e272390-8c398be037738dc042009320;Parent=94ae789b969f1cc5;Sampled=1
    logger.debug("Reading trace context from env var %s", raw_trace_id)
    if len(raw_trace_id) == 0:
        return None
    parts = raw_trace_id.split(";")
    if len(parts) != 3:
        return None
    root = parts[0].replace("Root=", "")
    parent = parts[1].replace("Parent=", "")
    sampled = parts[2].replace("Sampled=", "")
    if (
        len(root) == len(parts[0])
        or len(parent) == len(parts[1])
        or len(sampled) == len(parts[2])
    ):
        return None
    return {
        "parent_id": parent,
        "trace_id": root,
        "sampled": sampled,
        "source": TraceContextSource.XRAY,
    }


def generate_random_id():
    return binascii.b2a_hex(os.urandom(8)).decode("utf-8")


def build_segment(context, key, metadata):

    segment = json.dumps(
        {
            "id": generate_random_id(),
            "trace_id": context["trace_id"],
            "parent_id": context["parent_id"],
            "name": XraySubsegment.NAME,
            "start_time": time.time(),
            "end_time": time.time(),
            "type": "subsegment",
            "metadata": {
                XraySubsegment.NAMESPACE: {
                    key: metadata,
                }
            },
        }
    )
    return segment


def send_segment(key, metadata):
    host_port_tuple = get_xray_host_port(
        os.environ.get(XrayDaemon.XRAY_DAEMON_ADDRESS, "")
    )
    if host_port_tuple is None:
        return None
    context = parse_xray_header(
        os.environ.get(XrayDaemon.XRAY_TRACE_ID_HEADER_NAME, "")
    )
    if context is None:
        logger.debug(
            "Failed to create segment since it was not possible to get trace context from header"
        )
        return None

    # Skip adding segment, if the xray trace is going to be sampled away.
    if context["sampled"] == "0":
        logger.debug("Skipping sending metadata, x-ray trace was sampled out")
        return None
    segment = build_segment(context, key, metadata)
    segment_payload = build_segment_payload(segment)
    send(host_port_tuple, segment_payload)



import os
import logging
import base64
from datadog_lambda.extension import should_use_extension

logger = logging.getLogger(__name__)
KMS_ENCRYPTION_CONTEXT_KEY = "LambdaFunctionName"


def decrypt_kms_api_key(kms_client, ciphertext):
    from botocore.exceptions import ClientError

    """
    Decodes and deciphers the base64-encoded ciphertext given as a parameter using KMS.
    For this to work properly, the Lambda function must have the appropriate IAM permissions.

    Args:
        kms_client: The KMS client to use for decryption
        ciphertext (string): The base64-encoded ciphertext to decrypt
    """
    decoded_bytes = base64.b64decode(ciphertext)

    """
    When the API key is encrypted using the AWS console, the function name is added as an
    encryption context. When the API key is encrypted using the AWS CLI, no encryption context
    is added. We need to try decrypting the API key both with and without the encryption context.
    """
    # Try without encryption context, in case API key was encrypted using the AWS CLI
    function_name = os.environ.get("AWS_LAMBDA_FUNCTION_NAME")
    try:
        plaintext = kms_client.decrypt(CiphertextBlob=decoded_bytes)[
            "Plaintext"
        ].decode("utf-8")
    except ClientError:
        logger.debug(
            "Failed to decrypt ciphertext without encryption context, \
            retrying with encryption context"
        )
        # Try with encryption context, in case API key was encrypted using the AWS Console
        plaintext = kms_client.decrypt(
            CiphertextBlob=decoded_bytes,
            EncryptionContext={
                KMS_ENCRYPTION_CONTEXT_KEY: function_name,
            },
        )["Plaintext"].decode("utf-8")

    return plaintext


def init_api():
    if (
        not should_use_extension
        and not os.environ.get("DD_FLUSH_TO_LOG", "").lower() == "true"
    ):
        # Make sure that this package would always be lazy-loaded/outside from the critical path
        # since underlying packages are quite heavy to load
        # and useless when the extension is present
        from datadog import api

        if not api._api_key:
            import boto3

            DD_API_KEY_SECRET_ARN = os.environ.get("DD_API_KEY_SECRET_ARN", "")
            DD_API_KEY_SSM_NAME = os.environ.get("DD_API_KEY_SSM_NAME", "")
            DD_KMS_API_KEY = os.environ.get("DD_KMS_API_KEY", "")
            DD_API_KEY = os.environ.get(
                "DD_API_KEY", os.environ.get("DATADOG_API_KEY", "")
            )

            if DD_API_KEY_SECRET_ARN:
                api._api_key = boto3.client("secretsmanager").get_secret_value(
                    SecretId=DD_API_KEY_SECRET_ARN
                )["SecretString"]
            elif DD_API_KEY_SSM_NAME:
                api._api_key = boto3.client("ssm").get_parameter(
                    Name=DD_API_KEY_SSM_NAME, WithDecryption=True
                )["Parameter"]["Value"]
            elif DD_KMS_API_KEY:
                kms_client = boto3.client("kms")
                api._api_key = decrypt_kms_api_key(kms_client, DD_KMS_API_KEY)
            else:
                api._api_key = DD_API_KEY

        logger.debug("Setting DATADOG_API_KEY of length %d", len(api._api_key))

        # Set DATADOG_HOST, to send data to a non-default Datadog datacenter
        api._api_host = os.environ.get(
            "DATADOG_HOST", "https://api." + os.environ.get("DD_SITE", "datadoghq.com")
        )
        logger.debug("Setting DATADOG_HOST to %s", api._api_host)

        # Unmute exceptions from datadog api client, so we can catch and handle them
        api._mute = False



class StatsWriter:
    def distribution(self, metric_name, value, tags=[], timestamp=None):
        raise NotImplementedError()

    def flush(self):
        raise NotImplementedError()

    def stop(self):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['datadog_lambda']

package_data = \
{'': ['*']}

install_requires = \
['datadog>=0.41.0,<0.42.0', 'ddtrace>=0.58.1,<0.59.0', 'wrapt>=1.11.2,<2.0.0']

extras_require = \
{':python_version < "3.8"': ['typing_extensions>=4.0,<5.0',
                             'importlib_metadata>=1.0,<2.0'],
 u'dev': ['nose2>=0.9.1,<0.10.0',
          'httpretty>=0.9.7,<0.10.0',
          'boto3>=1.10.33,<2.0.0',
          'requests>=2.22.0,<3.0.0',
          'flake8>=3.7.9,<4.0.0']}

setup_kwargs = {
    'name': 'datadog-lambda',
    'version': '3.53.0',
    'description': 'The Datadog AWS Lambda Library',
    'long_description': '# datadog-lambda-python\n\n![build](https://github.com/DataDog/datadog-lambda-python/workflows/build/badge.svg)\n[![PyPI](https://img.shields.io/pypi/v/datadog-lambda)](https://pypi.org/project/datadog-lambda/)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/datadog-lambda)\n[![Slack](https://chat.datadoghq.com/badge.svg?bg=632CA6)](https://chat.datadoghq.com/)\n[![License](https://img.shields.io/badge/license-Apache--2.0-blue)](https://github.com/DataDog/datadog-lambda-python/blob/main/LICENSE)\n\nDatadog Lambda Library for Python (3.6, 3.7, 3.8, and 3.9) enables enhanced Lambda metrics, distributed tracing, and custom metric submission from AWS Lambda functions.\n\n**IMPORTANT NOTE:** AWS Lambda is expected to receive a [breaking change](https://aws.amazon.com/blogs/compute/upcoming-changes-to-the-python-sdk-in-aws-lambda/) on **March 31, 2021**. If you are using Datadog Python Lambda layer version 7 or below, please upgrade to the latest.\n\n## Installation\n\nFollow the [installation instructions](https://docs.datadoghq.com/serverless/installation/python/), and view your function\'s enhanced metrics, traces and logs in Datadog.\n\n## Custom Metrics\n\nOnce [installed](#installation), you should be able to submit custom metrics from your Lambda function.\n\nCheck out the instructions for [submitting custom metrics from AWS Lambda functions](https://docs.datadoghq.com/integrations/amazon_lambda/?tab=python#custom-metrics).\n\n## Tracing\n\nOnce [installed](#installation), you should be able to view your function\'s traces in Datadog, and your function\'s logs should be automatically connected to the traces.\n\nFor additional details on trace collection, take a look at [collecting traces from AWS Lambda functions](https://docs.datadoghq.com/integrations/amazon_lambda/?tab=python#trace-collection).\n\nFor additional details on trace and log connection, see [connecting logs and traces](https://docs.datadoghq.com/tracing/connect_logs_and_traces/python/).\n\nFor additional details on the tracer, check out the [official documentation for Datadog trace client](http://pypi.datadoghq.com/trace/docs/index.html).\n\n## Enhanced Metrics\n\nOnce [installed](#installation), you should be able to view enhanced metrics for your Lambda function in Datadog.\n\nCheck out the official documentation on [Datadog Lambda enhanced metrics](https://docs.datadoghq.com/integrations/amazon_lambda/?tab=python#real-time-enhanced-lambda-metrics).\n\n## Advanced Configurations\n\n### Handler Wrapper\n\nIn order to instrument individual invocations, the Datadog Lambda library needs to wrap around your Lambda handler function. This is usually achieved by setting your function\'s handler to the Datadog handler function (`datadog_lambda.handler.handler`) and setting the environment variable `DD_LAMBDA_HANDLER` with your original handler function to be called by the Datadog handler.\n\nIf this method doesn\'t work for you, instead of setting the handler and the `DD_LAMBDA_HANDLER` environment variable, you can apply the Datadog Lambda library wrapper in your function code like below:\n\n```python\nfrom datadog_lambda.wrapper import datadog_lambda_wrapper\n\n@datadog_lambda_wrapper\ndef my_lambda_handle(event, context):\n    # your function code\n```\n\n## Environment Variables\n\n### DD_FLUSH_TO_LOG\n\nSet to `true` (recommended) to send custom metrics asynchronously (with no added latency to your Lambda function executions) through CloudWatch Logs with the help of [Datadog Forwarder](https://github.com/DataDog/datadog-serverless-functions/tree/main/aws/logs_monitoring). Defaults to `false`. If set to `false`, you also need to set `DD_API_KEY` and `DD_SITE`.\n\n### DD_API_KEY\n\nIf `DD_FLUSH_TO_LOG` is set to `false` (not recommended), the Datadog API Key must be defined by setting one of the following environment variables:\n\n- DD_API_KEY - the Datadog API Key in plain-text, NOT recommended\n- DD_KMS_API_KEY - the KMS-encrypted API Key, requires the `kms:Decrypt` permission\n- DD_API_KEY_SECRET_ARN - the Secret ARN to fetch API Key from the Secrets Manager, requires the `secretsmanager:GetSecretValue` permission (and `kms:Decrypt` if using a customer managed CMK)\n- DD_API_KEY_SSM_NAME - the Parameter Name to fetch API Key from the Systems Manager Parameter Store, requires the `ssm:GetParameter` permission (and `kms:Decrypt` if using a SecureString with a customer managed CMK)\n\nYou can also supply or override the API key at runtime (not recommended):\n\n```python\n# Override DD API Key after importing datadog_lambda packages\nfrom datadog import api\napi._api_key = "MY_API_KEY"\n```\n\n### DD_SITE\n\nIf `DD_FLUSH_TO_LOG` is set to `false` (not recommended), you must set `DD_SITE`. Possible values are `datadoghq.com`, `datadoghq.eu`, `us3.datadoghq.com`, `us5.datadoghq.com`, and `ddog-gov.com`. The default is `datadoghq.com`.\n\n### DD_LOGS_INJECTION\n\nInject Datadog trace id into logs for [correlation](https://docs.datadoghq.com/tracing/connect_logs_and_traces/python/) if you are using a `logging.Formatter` in the default `LambdaLoggerHandler` by the Lambda runtime. Defaults to `true`.\n\n### DD_LOG_LEVEL\n\nSet to `debug` enable debug logs from the Datadog Lambda Library. Defaults to `info`.\n\n### DD_ENHANCED_METRICS\n\nGenerate enhanced Datadog Lambda integration metrics, such as, `aws.lambda.enhanced.invocations` and `aws.lambda.enhanced.errors`. Defaults to `true`.\n\n### DD_LAMBDA_HANDLER\n\nYour original Lambda handler.\n\n### DD_TRACE_ENABLED\n\nInitialize the Datadog tracer when set to `true`. Defaults to `false`.\n\n### DD_MERGE_XRAY_TRACES\n\nSet to `true` to merge the X-Ray trace and the Datadog trace, when using both the X-Ray and Datadog tracing. Defaults to `false`.\n\n### DD_TRACE_MANAGED_SERVICES (experimental)\n\nInferred Spans are spans that Datadog can create based on incoming event metadata.\nSet `DD_TRACE_MANAGED_SERVICES` to `true` to infer spans based on Lambda events.\nInferring upstream spans is only supported if you are using the [Datadog Lambda Extension](https://docs.datadoghq.com/serverless/libraries_integrations/extension/).\nDefaults to `true`.\nInfers spans for:\n\n- API Gateway REST events\n- API Gateway WebSocket events\n- HTTP API events\n- SQS\n- SNS (SNS messaged delivered via SQS are also supported)\n- Kinesis Streams (if data is a JSON string or base64 encoded JSON string)\n- EventBridge (custom events, where Details is a JSON string)\n- S3\n- DynamoDB\n\n## Opening Issues\n\nIf you encounter a bug with this package, we want to hear about it. Before opening a new issue, search the existing issues to avoid duplicates.\n\nWhen opening an issue, include the Datadog Lambda Library version, Python version, and stack trace if available. In addition, include the steps to reproduce when appropriate.\n\nYou can also open an issue for a feature request.\n\n## Contributing\n\nIf you find an issue with this package and have a fix, please feel free to open a pull request following the [procedures](CONTRIBUTING.md).\n\n## Community\n\nFor product feedback and questions, join the `#serverless` channel in the [Datadog community on Slack](https://chat.datadoghq.com/).\n\n## License\n\nUnless explicitly stated otherwise all files in this repository are licensed under the Apache License Version 2.0.\n\nThis product includes software developed at Datadog (https://www.datadoghq.com/). Copyright 2019 Datadog, Inc.\n',
    'author': 'Datadog, Inc.',
    'author_email': 'dev@datadoghq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/DataDog/datadog-lambda-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.0,<4',
}


setup(**setup_kwargs)
